<?php

/***********************************************************************

 設定は config.php ファイルをごらんください

************************************************************************/



// ---------------------------------------------------------------------


require_once('./inc/topology.php');

//define('DIFF_MAX', MOBIL_MAX + DEPTH_MAX + SPEED_MAX);

class GameSession {
	private function __construct() {
		$this->game_id = md5(uniqid(mt_rand(), true));
		$this->log = array();
		$this->score_ranking = array();
		$time = time();
		$this->players = array();
		$this->last_update = array(self::UPDATE_OBJECTS => $time, self::UPDATE_LOG => $time, self::UPDATE_RANKING => $time, self::UPDATE_CURRENT => $time);
		$this->putLog(GAME_NAME.'の開始');
	}
	
	const DMG_ALIVE = 0;
	const DMG_DEST = 1;
	
	const ATK_NONE = 0;
	const ATK_CUTLASS = 1;
	const ATK_SHIELD = 2;
	const ATK_MINE = 3;
	const ATK_TORPEDO = 4;
	const ATK_FALL = 5;
	
	const UPDATE_NONE = 0;
	const UPDATE_STATUS = 1;
	const UPDATE_OBJECTS = 2;
	const UPDATE_LOG = 3;
	const UPDATE_RANKING = 4;
	const UPDATE_CURRENT = 5;
	
	public $players, $objects, $game_id, $last_time, $input_start_time;
	private $current, $last_update, $log, $score_ranking;
	
	public function __destruct() {
		self::fileUnlock();
	}
	
	public function __wakeup() {
		
	}
	
	private static $datafile;
	public function save($update = true) {
		//$this->last_time = time();
		//$this->last_update = time();
		if ($update)
			$this->last_update[self::UPDATE_OBJECTS] = time();
		$handle = fopen(self::$datafile, 'w');
		fwrite($handle, serialize($this));
		fclose($handle);
	}
	
	public static function load() {
		self::$datafile = SAVE_DIR . FILE_NAME;
		self::fileLock(SAVE_DIR . '_' . FILE_NAME);
		$data = @file_get_contents(self::$datafile);
		if ($data)
			return unserialize($data);
		else
			return new GameSession(SAVE_DIR, FILE_NAME);
	}
	
	private static $lockfile;
	private static function fileLock($lockfile) {
		self::$lockfile = $lockfile;
		clearstatcache();
		if (is_file(self::$lockfile) and (time() - filemtime(self::$lockfile)) < 30) {
			for ($i = 0; is_file(self::$lockfile) and $i < 10; ++$i) {
				sleep(1);
				clearstatcache();
			}
		}
		touch(self::$lockfile);
	}
	
	private static function fileUnlock() {
		clearstatcache();
		if (self::$lockfile != false and is_file(self::$lockfile))
			unlink(self::$lockfile);
	}
	
	public function isLogin() {
		$id = (isset($_COOKIE[$this->game_id]))? $_COOKIE[$this->game_id]: '';
		
		$time = time();
		if ($this->current != null and (($this->input_start_time == 0 and $time - $this->last_time > INPUT_TIME_LIMIT) or ($this->input_start_time != 0 and $time - $this->input_start_time > INPUT_TIME_LIMIT))) {
			if ($time - $this->last_time > INPUT_TIME_LIMIT * 2) {
				foreach ($this->players as $id => $pl) {
					$this->putLog('制限時間オーバーにより'.$pl->rb->name.'が自壊');
					$this->damage($pl->rb->id, -1);
					$this->gameEnd($id);
				}
				$this->current = null;
				$this->gameNext($this->current);
			} elseif ($this->attack_list) {
				$this->gameNext($this->current);
			} else {
				$this->putLog('制限時間オーバーにより'.$this->players[$this->current]->rb->name.'が自壊');
				$this->damage($this->players[$this->current]->rb->id, -1);
				$this->gameEnd($this->current);
				$this->gameNext($this->current);
			}
		}
		
		if ($id != '' and isset($this->players[$id]) and $this->players[$id]->is_login)
			return $id;
		else
			return false;
	}
	
	public function isAlive($id) {
		return ($id != '' and isset($this->players[$id]) and $this->players[$id]->is_login);
	}
	
	private function putLog($msg) {
		if (LOG_MAX <= 0)
			return;
		$this->last_update[self::UPDATE_LOG] = time();
		array_unshift($this->log, array(time(), $msg));
		while (count($this->log) > LOG_MAX)
			array_pop($this->log);
	}
	
	public function login($name, $color) {
		if ($this->isLogin() == false) {
			if (count($this->players) >= PLAYER_MAX) {
				$count = 0;
				foreach ($this->players as $player) {
					if ($player->is_login)
						++$count;
				}
				if ($count >= PLAYER_MAX)
					return '現在空きがありません';
			}
			if ($name == '')
				return '名前を入力してください';
		
			$id = uniqid(mt_rand(), true);
			
			$time = time();
			setcookie($this->game_id, $id, $time+86400);
			
			$m = mt_rand(MOBIL_RANGE_MIN, MOBIL_RANGE_MAX);
			$d = mt_rand(DEPTH_RANGE_MIN, DEPTH_RANGE_MAX);
			$s = mt_rand(SPEED_RANGE_MIN, SPEED_RANGE_MAX);
			
			$color_vals = explode(',', $color);
			for ($i = 0; $i < 3; ++$i) {
				if ($color_vals[i] < COLOR_RANGE_MIN)
					$color_vals[i] = COLOR_RANGE_MIN;
				elseif ($color_vals[i] > COLOR_RANGE_MAX)
					$color_vals[i] = COLOR_RANGE_MAX;
			}
			
			$this->players[$id] = new Player($id, $name, $color_vals[0], $color_vals[1], $color_vals[2], $m, $d, $s);
			$this->objects[$this->players[$id]->rb->id] = &$this->players[$id]->rb;
			
			$this->putLog($name.'さんが'.GAME_NAME.'への介入を開始');
			if (count($this->players) == 1) {
				//$this->game_next($id);
				reset($this->objects);
				$this->current = $id;
				$this->last_time = $time;
				$this->last_update[self::UPDATE_CURRENT] = $time;
			}
			
			$this->save();
			
			return null;
		}
		return 'ページを読み込み直してください';
	}
	
	function gameLogout($id) {
		if (!isset($this->players[$id]))
			return;
		$this->putLog($this->players[$id]->name.'さんが'.GAME_NAME.'への介入を終了');
		//$this->damage($this->players[$id]->rb->id, -1);
		if (isset($_COOKIE[$this->game_id]) and $_COOKIE[$this->game_id] == $id)
			setcookie($this->game_id, '', time() - 3600);
		
		$this->players[$id]->is_login = false;
		
		if ($this->current == $id) {
			$this->gameNext($id);
		} else {
			$this->save();
		}
	}
	
	function gameEnd($id) {
		if (!isset($this->players[$id]))
			return;
		if (isset($this->objects[$this->players[$id]->rb->id]))
			unset($this->objects[$this->players[$id]->rb->id]);
		
		unset($this->players[$id]);
		if (isset($_COOKIE[$this->game_id]) and $_COOKIE[$this->game_id] == $id)
			setcookie($this->game_id, '', time() - 3600);
	}
	
	public function gameData($id, $last_update) {
		if ($this->last_update[self::UPDATE_OBJECTS] <= $last_update)
			return '0';
		
		$time = time();
		$result = array($time);
		
		$current_num = 0;
		if ($this->last_update[self::UPDATE_OBJECTS] > $last_update) {
			$result[] = self::UPDATE_OBJECTS;
			
			$current = $this->players[$this->current]->rb->id;
			$i = 0;
			foreach ($this->objects as $obj) {
				if ($current == $obj->id)
					$current_num = $i;
				
				$obj_data = array(($obj->pid == $id)? 1: 0, implode(',', $obj->color));
				if ($obj instanceof RoundBuckler) {
					$obj_data[] = 0;
					$obj_data[] = $obj->durab;
					$obj_data[] = implode(',', $obj->get());
					$obj_data[] = $this->players[$obj->pid]->score;
					
					if ($obj->pid == $id) {
						$obj_data[] = $obj->shield;
						$obj_data[] = $this->players[$obj->pid]->mine;
						$obj_data[] = $this->players[$obj->pid]->torpedo;
					} else {
						$obj_data[] = '0,0,0';
					}
				} else {
					if ($obj instanceof Torpedo)
						$obj_data[] = 2;
					else
						$obj_data[] = 1;
					$obj_data[] = '--';
					$obj_data[] = implode(',', $obj->get());
					$obj_data[] = '--,0,0,0';
				}
				$obj_data[] = $obj->name;
				$result[] = implode(',', $obj_data);
				
				++$i;
			}
			
			$result[] = self::UPDATE_OBJECTS;
		}
		
		if ($this->last_update[self::UPDATE_CURRENT] > $last_update) {
			$result[] = self::UPDATE_CURRENT;
			
			if ($this->current != $id) {
				if ($this->input_start_time == 0) {
					$limit = -1;
				} else {
					$limit = INPUT_TIME_LIMIT - ($time - $this->input_start_time);
					if ($limit < 0)
						$limit = 0;
				}
				$result[] = '0,'.$current_num.','.$limit;
			} elseif (!$this->attack_list) {
				setcookie($this->game_id, $id, $time+86400);
				
				$result[] = '1,'.$current_num.','.($this->input_start_time == 0? INPUT_TIME_LIMIT: (INPUT_TIME_LIMIT - ($time - $this->input_start_time)));
				
				$act_list = $this->players[$id]->rb->getActionList();
				foreach ($act_list as $num => $action) {
					$act_topology = RoundBuckler::$action_commands[$action][CMDS_TOPOLOGY];
					$topology = $this->players[$id]->rb->compare($act_topology)->get();
					$str = $num.','.$action.','.implode(',', $act_topology).','.implode(',', $topology).':';
					foreach ($this->players[$id]->rb->getActionList($action, $topology) as $next_action) {
						$str .= implode(',', RoundBuckler::$action_commands[$next_action][CMDS_TOPOLOGY]).',';
					}
					$result[] = $str;
				}
				
				if ($this->input_start_time == 0) {
					$this->input_start_time = $time;
					$this->last_update[self::UPDATE_CURRENT] = $time;
					$this->save(false);
				}
			} else {
				$limit = INPUT_TIME_LIMIT - ($time - $this->input_start_time);
				if ($limit < 0)
					$limit = 0;
				$result[] = '2,'.$current_num.','.$limit;
				
				foreach ($this->attack_list as $num => $action) {
					$result[] = $num.','.$action;
				}
			}
			
			$result[] = self::UPDATE_CURRENT;
		}

		
		if ($this->last_update[self::UPDATE_LOG] > $last_update and $last_update > 0) {
			$result[] = self::UPDATE_LOG;
			
			$result_log = array();
			foreach ($this->log as $log) {
				if ($log[0] > $last_update)
					array_unshift($result_log, $log[1]);
			}
			$result = array_merge($result, $result_log);
			
			$result[] = self::UPDATE_LOG;
		}
		
		if ($this->last_update[self::UPDATE_RANKING] > $last_update and $last_update > 0) {
			$result[] = self::UPDATE_RANKING;
			
			foreach ($this->score_ranking as $kill => $names) {
				foreach ($names as $name)
					$result[] = $kill.','.$name;
			}
			
			$result[] = self::UPDATE_RANKING;
		}
		
		return implode("\n", $result);
	}
	
	function damage($id, $p) {
		$dest = ($p < 0);
		if ($dest)
			$p = $this->objects[$id]->durab;
		$pid = $this->objects[$id]->pid;
		
		$this->objects[$id]->durab -= $p;
		if ($this->objects[$id]->durab <= 0) {
			if ($dest)
				unset($this->objects[$id]);
			$this->players[$pid]->id = '';
			foreach ($this->players[$pid]->mine_torpedo_list as $mid => $mine)
				$this->objects[$mid]->name = substr($mine->name, 0, strpos($mine->name, '@'));
			
			if (KILL_SCORE_RANKING > 0 and $this->players[$pid]->score > 0) {
				$score = array_keys($this->score_ranking);
				if (count($score) < KILL_SCORE_RANKING or $this->players[$pid]->score >= $min = min($score)) {
					$s = $this->players[$pid]->score;
					if (isset($this->score_ranking[$s])) {
						array_unshift($this->score_ranking[$s], $this->players[$pid]->name);
						if (count($this->score_ranking[$s]) > KILL_SCORE_RANKING)
							array_pop($this->score_ranking[$s]);
					} else {
						$this->score_ranking[$s] = array($this->players[$pid]->name);
						krsort($this->score_ranking);
						if (count($this->score_ranking) > KILL_SCORE_RANKING)
							unset($this->score_ranking[$min]);
					}
					$this->last_update[self::UPDATE_RANKING] = time();
				}
			}
			return self::DMG_DEST;
		}
		if ($this->players[$pid]->auto_repair == 0)
			$this->players[$pid]->auto_repair = (AUTO_REPAIR_TURN <= 0)? 0: AUTO_REPAIR_TURN;
		$this->players[$pid]->morale -= 2;
		if ($this->players[$pid]->morale < 50)
			$this->players[$pid]->morale = 50;
		return self::DMG_ALIVE;
	}
	
	private function gameNext($id) {
		if ($id != $this->current)
			return false;
		
		$this->target_list = null;
		$this->attack_list = null;
		
		if (count($this->players) == 0) {
			$this->current = null;
			$this->last_update[self::UPDATE_CURRENT] = time();
			
			$this->last_time = time();
			$this->input_start_time = 0;
			
			$this->save();
			return true;
		}
		
		if (isset($this->players[$id]))
			$id = $this->players[$id]->rb->id;
		reset($this->objects);
		$objects_copy = $this->objects;
		while (list($key, $value) = each($objects_copy)) {
			if ($key == $id)
				break;
		}
		$next = $id;
		$loop = 0;
		do {
			if (count($this->players) == 0)
				break;
			list($next, $value) = each($objects_copy);
			if ($value == false) {
				$id_list = array();
				foreach ($this->objects as $id => $obj)
					$id_list[$id] = $obj->turnOrder();
				$id_list = array_reverse($id_list);
				arsort($id_list);
				
				$tmp = array();
				foreach ($id_list as $id => $v)
					$tmp[$id] = &$this->objects[$id];
				$this->objects = $tmp;
				$objects_copy = $this->objects;
				
				unset($id_list);
				++$loop;
			} elseif (!isset($this->objects[$next])) {
				continue;
			} elseif ($value instanceof Torpedo) {
				$pri = array(0 => -1, 1 => false, 2 => false);
				foreach ($this->players as $pid => $player) {
					if ($pid != $value->pid) {
						$a_d = $value->absDiffTopology($player->rb);
						$sum = $a_d->sum();
						if (($sum <= $pri[0]) or $pri[0] == -1 and ($a_d->mobil < 10 or $a_d->depth < 10 or $a_d->speed < 10)) {
							if ($sum < $pri[0] or $pri[0] == -1 or $a_d->depth < $pri[1]->depth or ($a_d->depth == $pri[1]->depth and $a_d->speed < $pri[1]->speed) or ($a_d->mobil == $pri[1]->mobil and $a_d->depth == $pri[1]->depth and $a_d->speed == $pri[1]->speed and mt_rand(0, 1))) {
								$pri[0] = $sum;
								$pri[1] = $a_d;
								$pri[2] = $player->rb->id;
							}
						}
					}
				}
				
				$acts = null;
				if ($pri[1]) {
					$act_list = $value->getActionList();
					$acts = array(array(-1, false, false), array(-1, false, false));
					
					if ($pri[0] == 3 and $pri[1]->depth == 1 and $pri[1]->speed == 1 and $pri[1]->mobil == 1) {
						$diff = $value->diffTopology($this->objects[$pri[2]]);
						$sum = $diff->sum();
						if ($sum > -3) {
							if ($pri[1]->depth == 1 and in_array(ACT_DEP_M, $act_list))
								$act = ACT_DEP_M;
							elseif ($pri[1]->speed == 1 and in_array(ACT_SPD_M, $act_list))
								$act = ACT_SPD_M;
							elseif ($pri[1]->mobil == 1 and in_array(ACT_MBL_M, $act_list))
								$act = ACT_MBL_M;
							else
								$act = ACT_STOP;
						} else {
							if (in_array(ACT_DS_PM, $act_list))
								$act = ACT_DS_PM;
							elseif (in_array(ACT_SM_PM, $act_list))
								$act = ACT_SM_PM;
							elseif (in_array(ACT_MD_PM, $act_list))
								$act = ACT_MD_PM;
							else
								$act = ACT_STOP;
						}
						$acts[0][0] = 4;
						$acts[0][1] = 4;
						$acts[0][2] = $act;
						$n = 0;
					} else {
						//$diff = $value[PL_BODY]->diff($this->players[$pri[2]][PL_BODY]);
						$max = $pri[1]->max();
						if ($max == $pri[1]->depth)
							$key = Topology::depth_key;
						elseif ($max == $pri[1]->speed)
							$key = Topology::speed_key;
						else
							$key = Topology::mobil_key;
						
						foreach ($act_list as $act_num => $act) {
							$base = $value->compare(Torpedo::$action_commands[$act][CMDS_TOPOLOGY]);
							$diff = new Topology(abs($this->objects[$pri[2]]->mobil - $base->mobil), abs($this->objects[$pri[2]]->depth - $base->depth), abs($this->objects[$pri[2]]->speed - $base->speed));
							$sum = $diff->sum();
							if ($sum < $acts[0][0] or $acts[0][0] == -1) {
								$acts[0][0] = $sum;
								$acts[0][1] = $sum;
								$acts[0][2] = $act;
								if ($sum <= 2)
									break;
							}
							if ($diff->topology[$key] < $acts[1][0] or $acts[1][0] == -1) {
								$acts[1][0] = $diff->topology[$key];
								$acts[1][1] = $sum;
								$acts[1][2] = $act;
							}
						}
						
						$n = ($acts[0][0] <= 2 or $acts[0][0] < $acts[1][0] or $acts[1][0] == 10)? 0: 1;
					}
					
					$move_action = $acts[$n][2];
				} else {
					$act_list = $value->getActionList();
					$move_action = $act_list[array_rand($act_list)];
				}
				
				$move_msg = $this->objects[$next]->move($move_action);
				$this->putLog($this->objects[$next]->name.'が'.$move_msg);
				
				$targets = array();
				$temp = $this->objects;
				foreach ($temp as $obj) {
					if ($obj instanceof Mine and $obj->pid != $value->pid) {
						$a_diff = $value->absDiffTopology($obj);
						if ($a_diff->sum() <= 2 and $a_diff->depth <= 1 and $a_diff->mobil <= 1 and $a_diff->speed <= 1)
							$targets[] = $obj->id;
					}
				}
					
				if (count($targets) > 0) {
					$target = $targets[array_rand($targets)];
					if (7 == mt_rand(1, 10)) {
						$this->putLog($value->name.'は'.$this->objects[$target]->name.'への反応に失敗');
					} else {
						$this->putLog($value->name.'が'.$this->objects[$target]->name.'に誤反応し破壊');
						if ($this->objects[$target]->pid != '' and isset($this->players[$this->objects[$target]->pid]))
							unset($this->players[$this->objects[$target]->pid]->mine_torpedo_list[$target]);
						unset($this->objects[$target]);
					}
					if ($value->pid != '' and isset($this->players[$value->pid]))
						unset($this->players[$value->pid]->mine_torpedo_list[$next]);
					unset($this->objects[$next]);
				} elseif ($acts and $acts[$n][1] <= 2) {
					if (7 == mt_rand(1, 10)) {
						$this->putLog($value->name.'は'.$this->objects[$pri[2]]->name.'への反応に失敗');
					} else {
						$dmg = mt_rand(TORPEDO_DAMAGE_MIN, TORPEDO_DAMAGE_MAX);
						$this->putLog($value->name.'が'.$this->objects[$pri[2]]->name.'に反応！ '.$dmg.'ダメージ');
						$pid = $this->objects[$pri[2]]->pid;
						$result = $this->damage($pri[2], $dmg);
						if (self::DMG_ALIVE != $result) {
							$this->putLog($value->name.'が'.$this->objects[$pri[2]]->name.'を破壊');
							if ($value->pid != '' and isset($this->players[$value->pid])) {
								$this->players[$value->pid]->score += 1;
								$this->players[$value->pid]->morale += 5;
								if ($this->players[$value->pid]->morale > 150)
									$this->players[$value->pid]->morale = 150;
							}
							$this->gameEnd($pid);
						}
					}
					if ($value->pid != '' and isset($this->players[$value->pid]))
						unset($this->players[$value->pid]->mine_torpedo_list[$next]);
					unset($this->objects[$next]);
				} else {
					if($this->objects[$next]->durab > 0 and --$this->objects[$next]->durab == 0) {
						if($value->pid != '' and isset($this->players[$value->pid]))
							unset($this->players[$value->pid]->mine_torpedo_list[$next]);
						$this->putLog($value->name.'の寿命が切れた');
						unset($this->objects[$next]);
					}
				}
			} elseif ($value instanceof Mine) {
				$pri = array(array(), array());
				foreach ($this->players as $player) {
					if ($player->id != $value->pid) {
						$sum = $value->absDiffTopology($player->rb)->sum();
						if ($sum <= 2)
							$pri[$sum][] = $player->rb->id;
					}
				}
				$target = false;
				for ($n = 0; $n <= 2; ++$n) {
					if (isset($pri[$n]) and count($pri[$n]) > 0) {
						$i = array_rand($pri[$n]);
						$target = $pri[$n][$i];
						break;
					}
				}
				if ($target) {
					if (7 == mt_rand(1, 15)) {
						$this->putLog($value->name.'は'.$this->objects[$target]->name.'への反応に失敗');
					} else {
						$dmg = mt_rand(MINE_DAMAGE_MIN, MINE_DAMAGE_MAX);
						$this->putLog($value->name.'が'.$this->objects[$target]->name.'に反応！ '.$dmg.'ダメージ');
						$pid = $this->objects[$target]->pid;
						$result = $this->damage($target, $dmg);
						if (self::DMG_ALIVE != $result) {
							$this->putLog($value->name.'が'.$this->objects[$target]->name.'を破壊');
							if ($value->pid != '' and isset($this->players[$value->pid])) {
								$this->players[$value->pid]->score += 1;
								$this->players[$value->pid]->morale += 5;
								if ($this->players[$value->pid]->morale > 150)
									$this->players[$value->pid]->morale = 150;
							}
							$this->gameEnd($pid);
						}
					}
					if ($value->pid != '' and isset($this->players[$value->pid]))
						unset($this->players[$value->pid]->mine_torpedo_list[$next]);
					unset($this->objects[$next]);
				} else {
					if ($value->durab < 0 or --$this->objects[$next]->durab > 0) {
						$move_msg = $this->objects[$next]->move();
						//$this->putLog($this->objects[$next]->name.'が'.$move_msg);
					} else {
						if ($value->pid != '' and isset($this->players[$value->pid]))
							unset($this->players[$value->pid]->mine_torpedo_list[$next]);
						$this->putLog($value->name.'の寿命が切れた');
						unset($this->objects[$next]);
					}
				}
			} else {
				if ($this->objects[$next]->shield == RoundBuckler::shield_max) {
					$pid = $this->objects[$next]->pid;
					$result = $this->damage($next, SHIELD_MAX_DAMAGE);
					if (self::DMG_ALIVE != $result) {
						$this->putLog($this->objects[$next]->name.'がエネルギー不足により自壊！');
						$this->gameEnd($pid);
					} else {
						break;
					}
				} elseif (!$this->players[$this->objects[$next]->pid]->is_login) {
					$pid = $this->objects[$next]->pid;
					$this->putLog($this->objects[$next]->name.'が離脱を完了');
					$this->damage($next, -1);
					$this->gameEnd($pid);
				} else {
					break;
				}
			}
		} while ($loop < 2);
		
		if ($next != '' and isset($this->objects[$next])) {
			$this->current = $this->objects[$next]->pid;
			if ($this->players[$this->current]->auto_repair > 0 and AUTO_REPAIR_PT > 0 and $this->players[$this->current]->rb->shield != RoundBuckler::shield_max and --$this->players[$this->current]->auto_repair == 0) {
				$this->players[$this->current]->rb->durab += AUTO_REPAIR_PT;
				if ($this->players[$this->current]->rb->durab >= RoundBuckler::durab_max) {
					$this->players[$this->current]->rb->durab = RoundBuckler::durab_max;
					$this->players[$this->current]->auto_repair = 0;
				} else {
					$this->players[$this->current]->auto_repair = (AUTO_REPAIR_TURN <= 0)? 0: AUTO_REPAIR_TURN;
				}
			}
			if ($this->players[$this->current]->auto_add_mine > 0 and --$this->players[$this->current]->auto_add_mine == 0) {
				if (++$this->players[$this->current]->mine >= MINE_MAX) {
					$this->players[$this->current]->mine = MINE_MAX;
					$this->players[$this->current]->auto_add_mine = 0;
				} else {
					$this->players[$this->current]->auto_add_mine = (AUTO_ADD_MINE_TURN <= 0)? 0: AUTO_ADD_MINE_TURN;
				}
			}
			if ($this->players[$this->current]->auto_add_torpedo > 0 and --$this->players[$this->current]->auto_add_torpedo == 0) {
				if (++$this->players[$this->current]->torpedo >= TORPEDO_MAX) {
					$this->players[$this->current]->torpedo = TORPEDO_MAX;
					$this->players[$this->current]->auto_add_torpedo = 0;
				} else {
					$this->players[$this->current]->auto_add_torpedo = (AUTO_ADD_TORPEDO_TURN <= 0)? 0: AUTO_ADD_TORPEDO_TURN;
				}
			}
		} else {
			$this->current = null;
		}
		
		$time = time();
		$this->last_time = $time;
		$this->input_start_time = 0;
		$this->last_update[self::UPDATE_CURRENT] = $time;
		
		$this->save();
		return true;
	}
	
	private $target_list, $attack_list;
	public function moveAction($id, $action_num) {
		if ($id != $this->current or $this->attack_list)
			return false;
		
		//$is_depth_over = $this->players[$id]->rb->isDepthOver($action);
		$is_depth_over = false;
		$move_msg = $this->players[$id]->rb->move($is_depth_over, $action_num);
		if (!$move_msg)
			return false;
		$this->putLog($this->players[$id]->rb->name.'が'.$move_msg);
		if ($is_depth_over) {
			$dmg = mt_rand(CRASH_DAMAGE_MIN, CRASH_DAMAGE_MAX);
			$result = $this->damage($this->players[$id]->rb->id, $dmg);
			if ($result != self::DMG_ALIVE) {
				$this->putLog($this->players[$id]->rb->name.'は海底に激突し自壊しました');
				$this->gameEnd($id);
				$this->gameNext($id);
				return true;
			} else {
				$this->putLog($this->players[$id]->rb->name.'が海底に激突！ '.$dmg.'ダメージ');
			}
		}
		
		$this->target_list = array(self::ATK_CUTLASS => array(array(), array(), array(), array()), self::ATK_SHIELD => array(array(), array(), array(), array(), array(), array()));
		$is_cutlass = false;
		$is_shield = false;
		foreach ($this->objects as $obj) {
			if ($obj->pid == $id)
				continue;
			
			$abs_diff = $this->players[$id]->rb->absDiffTopology($obj);
			$sum = $abs_diff->sum();
			if ($sum <= 1) {
				$is_cutlass = true;
				$this->target_list[self::ATK_CUTLASS][$sum + intval($obj instanceof RoundBuckler)][] = $obj->id;
				if ($this->players[$id]->rb->shield == RoundBuckler::shield_max or ($this->players[$id]->rb->shield == RoundBuckler::shield_on and $sum == 0)) {
					$is_shield = true;
					$this->target_list[self::ATK_SHIELD][$sum + intval($obj instanceof RoundBuckler)][] = $obj->id;
				}
			} elseif ($this->players[$id]->rb->shield == RoundBuckler::shield_max) {
				$diff = $this->players[$id]->rb->diffTopology($obj);
				if ($diff->mobil == -1 and $diff->depth == -1  and $diff->speed == -1) {
					$is_shield = true;
					$this->target_list[self::ATK_SHIELD][4 + intval($obj instanceof RoundBuckler)][] = $obj->id;
				}
			}
		}
		
		$result = array();
		$result[self::ATK_NONE] = '何もしない';
		if ($is_cutlass)
			$result[self::ATK_CUTLASS] = '切りつける';
		if ($is_shield)
			$result[self::ATK_SHIELD] = 'シールドアタック';
		if ($this->players[$id]->mine > 0)
			$result[self::ATK_MINE] = '機雷射出';
		if ($this->players[$id]->torpedo > 0)
			$result[self::ATK_TORPEDO] = '魚雷射出';
		$result[self::ATK_FALL] = '離脱';
		
		$this->attack_list = $result;
		
		$this->last_update[self::UPDATE_CURRENT] = time();
		$this->save();
		
		return true;
	}
	
	public function attackAction($id, $action) {
		if ($id != $this->current or !$this->attack_list)
			return;
	
		//if ($comment != '')
		//	$this->putLog($this->players[$id]->name.'「'.$comment.'」');
		
		if ($action == self::ATK_MINE) {
			$mine_id = uniqid('mine');
			$mine = new Mine($mine_id, $this->players[$id]->id, '機雷['.++$this->players[$id]->used_mines.']@'.$this->players[$id]->name, $this->players[$id]->color, $this->players[$id]->rb->mobil, $this->players[$id]->rb->depth, $this->players[$id]->rb->speed);
			$mine->addInt(mt_rand(-1, 1), mt_rand(-1, 1), mt_rand(-1, 1));
			$this->players[$id]->mine_torpedo_list[$mine_id] = &$mine;
			--$this->players[$id]->mine;
			$tmp = array();
			foreach ($this->objects as $key => $obj) {
				$tmp[$key] = &$this->objects[$key];
				if ($key == $this->players[$id]->rb->id)
					$tmp[$mine_id] = &$mine;
			}
			$this->objects = $tmp;
			unset($tmp);
			if ($this->players[$id]->auto_add_mine == 0)
				$this->players[$id]->auto_add_mine = (AUTO_ADD_MINE_TURN <= 0)? 0: AUTO_ADD_MINE_TURN;
			$this->putLog($this->players[$id]->rb->name.'が'.$mine->name.'を射出');
		} elseif ($action == self::ATK_TORPEDO) {
			$torpedo_id = uniqid('torpedo');
			$torpedo = new Torpedo($torpedo_id, $this->players[$id]->id, '魚雷['.++$this->players[$id]->used_torpedos.']@'.$this->players[$id]->name, $this->players[$id]->color, $this->players[$id]->rb->mobil, $this->players[$id]->rb->depth, $this->players[$id]->rb->speed);
			$this->players[$id]->mine_torpedo_list[$torpedo_id] = &$torpedo;
			--$this->players[$id]->torpedo;
			$tmp = array();
			foreach ($this->objects as $key => $obj) {
				$tmp[$key] = &$this->objects[$key];
				if ($key == $this->players[$id]->rb->id)
					$tmp[$torpedo_id] = &$torpedo;
			}
			$this->objects = $tmp;
			unset($tmp);
			if ($this->players[$id]->auto_add_torpedo == 0)
				$this->players[$id]->auto_add_torpedo = (AUTO_ADD_TORPEDO_TURN <= 0)? 0: AUTO_ADD_TORPEDO_TURN;
			$this->putLog($this->players[$id]->rb->name.'が'.$torpedo->name.'を射出');
		} elseif ($action == self::ATK_CUTLASS) {
			for ($i = 0; $i < 4; ++$i) {
				if (count($this->target_list[self::ATK_CUTLASS][$i]) > 0) {
					$targets = $this->target_list[self::ATK_CUTLASS][$i];
					shuffle($targets);
					foreach ($targets as $target) {
						if (isset($this->objects[$target])) {
							if (7 == mt_rand(1, 10)) {
								$this->putLog($this->players[$id]->rb->name.'が攻撃を失敗');
								break 2;
							}
							if ($this->objects[$target] instanceof Mine) {
								$this->putLog($this->players[$id]->rb->name.'が'.$this->objects[$target]->name.'を破壊');
								if ($this->objects[$target]->pid != '' and isset($this->players[$this->objects[$target]->pid]))
									unset($this->players[$this->objects[$target]->pid]->mine_torpedo_list[$target]);
								unset($this->objects[$target]);
							} else {
								$dmg = (int)(mt_rand(CUTLASS_DAMAGE_MIN, CUTLASS_DAMAGE_MAX) * ($this->players[$id]->morale / 100));
								$this->putLog($this->players[$id]->rb->name.'が'.$this->objects[$target]->name.'へカトラスを振るう！ '.$dmg.'ダメージ');
								$pid = $this->objects[$target]->pid;
								$result = $this->damage($target, $dmg);
								if (self::DMG_ALIVE != $result) {
									$this->putLog($this->players[$id]->rb->name.'が'.$this->objects[$target]->name.'を破壊');
									$this->players[$id]->score += 1;
									$this->players[$id]->morale += 5;
									$this->gameEnd($pid);
								} else {
									$this->players[$id]->morale += 1;
								}
								if ($this->players[$id]->morale > 150)
									$this->players[$id]->morale = 150;
							}
							break 2;
						}
					}
				}
			}
		} elseif ($action == self::ATK_SHIELD) {
			for ($i = 0; $i < 4; ++$i) {
				if (count($this->target_list[self::ATK_SHIELD][$i]) > 0) {
					$targets = $this->target_list[self::ATK_SHIELD][$i];
					$end = false;
					$this->putLog($this->players[$id]->rb->name.'がシールドアタック！ ');
					foreach ($targets as $target) {
						if (isset($this->objects[$target])) {
							$end = true;
							$this->players[$id]->rb->setTopology($this->objects[$target]);
							$this->putLog($this->players[$id]->rb->name.'が'.$this->objects[$target]->name.'を破壊');
							if ($this->objects[$target] instanceof Mine) {
								if ($this->objects[$target]->pid != '' and isset($this->players[$this->objects[$target]->pid]))
									unset($this->players[$this->objects[$target]->pid]->mine_torpedo_list[$target]);
								unset($this->objects[$target]);
							} else {
								$pid = $this->objects[$target]->pid;
								$this->damage($target, -1);
								$this->players[$id]->score += 1;
								$this->players[$id]->morale += 5;
								if($this->players[$id]->morale > 150)
									$this->players[$id]->morale = 150;
								$this->gameEnd($pid);
							}
						}
					}
					if ($end)
						break;
				}
			}
		} elseif ($action == self::ATK_FALL) {
			$this->gameLogout($id);
		}
		$this->gameNext($id);
		//$this->session_save();
	}
	
	public function playersList() {
		$players = array();
		foreach ($this->players as $player) {
			if ($player->is_login)
				$players[] = $player;
		}
		return $players;
	}
	
	public function actionOrderList() {
		return $this->objects;
	}
	
	public function gameLog() {
		return LOG_MAX > 0? $this->log: null;
	}
	
	public function scoreRanking() {
		return KILL_SCORE_RANKING > 0? $this->score_ranking: null;
	}
	
	public function getName($id) {
		return isset($this->players[$id])? $this->players[$id]->name: null;
	}
	
	public function getColor($id) {
		return isset($this->players[$id])? $this->players[$id]->color: null;
	}
}

class Player {
	public function __construct($id, $name, $r, $g, $b, $m, $d, $s) {
		$this->id = $id;
		$this->name = $name;
		$this->red = $r;
		$this->green = $g;
		$this->blue = $b;
		$this->color = array(&$this->red, &$this->green, &$this->blue);
		$this->rb = new RoundBuckler(uniqid('rb'), $this->id, $name.'機', $this->color, $m, $d, $s);
		$this->mine = MINE_MAX;
		$this->torpedo = TORPEDO_MAX;
		$this->morale = 100;
		$this->score = 0;
		$this->auto_repair = 0;
		$this->auto_add_mine = 0;
		$this->auto_add_torpedo = 0;
		$this->used_mines = 0;
		$this->used_torpedos = 0;
		$this->mine_torpedo_list = array();
		$this->is_login = true;
	}
	
	public $name, $id, $color, $red, $green, $blue, $rb, $mine, $torpedo, $morale, $score, $auto_repair, $auto_add_mine, $auto_add_torpedo, $used_mines, $used_torpedos, $mine_torpedo_list, $is_login;
}

?>